@@ -15,8 +15,8 @@ from hgext import record
 from tortoisehg.util import hglib
 from tortoisehg.util.patchctx import patchctx
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, thgrepo, qscilib, lexers, wctxactions
-from tortoisehg.hgqt import filelistmodel, filelistview, fileview
+from tortoisehg.hgqt import qtlib, thgrepo, qscilib, lexers, visdiff, revert
+from tortoisehg.hgqt import filelistmodel, filelistview, filedata
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -36,10 +36,13 @@ class ChunksWidget(QWidget):
     fileModelEmpty = pyqtSignal(bool)
     fileModified = pyqtSignal()
 
-    def __init__(self, repo, parent):
+    contextmenu = None
+
+    def __init__(self, repo, parent, multiselectable):
         QWidget.__init__(self, parent)
 
         self.repo = repo
+        self.multiselectable = multiselectable
         self.currentFile = None
 
         layout = QVBoxLayout(self)
@@ -53,9 +56,12 @@ class ChunksWidget(QWidget):
         self.splitter.setChildrenCollapsible(False)
         self.layout().addWidget(self.splitter)
 
-        self.filelist = filelistview.HgFileListView(self)
-        self.filelistmodel = filelistmodel.HgFileListModel(self.repo, self)
+        self.filelist = filelistview.HgFileListView(repo, self, multiselectable)
+        self.filelistmodel = filelistmodel.HgFileListModel(self)
         self.filelist.setModel(self.filelistmodel)
+        self.filelist.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.filelist.customContextMenuRequested.connect(self.menuRequest)
+        self.filelist.doubleClicked.connect(self.vdiff)
 
         self.fileListFrame = QFrame(self.splitter)
         self.fileListFrame.setFrameShape(QFrame.NoFrame)
@@ -71,18 +77,71 @@ class ChunksWidget(QWidget):
         self.diffbrowse.linkActivated.connect(self.linkActivated)
         self.diffbrowse.chunksSelected.connect(self.chunksSelected)
 
-        self.filelist.fileRevSelected.connect(self.displayFile)
+        self.filelist.fileSelected.connect(self.displayFile)
         self.filelist.clearDisplay.connect(self.diffbrowse.clearDisplay)
 
         self.splitter.setStretchFactor(0, 0)
         self.splitter.setStretchFactor(1, 3)
         self.timerevent = self.startTimer(500)
 
+        self._actions = {}
+        for name, desc, icon, key, tip, cb in [
+            ('diff', _('Visual Diff'), 'visualdiff', 'Ctrl+D',
+              _('View file changes in external diff tool'), self.vdiff),
+            ('edit', _('Edit Local'), 'edit-file', 'Shift+Ctrl+E',
+              _('Edit current file in working copy'), self.editCurrentFile),
+            ('revert', _('Revert to Revision'), 'hg-revert', 'Alt+Ctrl+T',
+              _('Revert file(s) to contents at this revision'),
+              self.revertfile),
+            ]:
+            act = QAction(desc, self)
+            if icon:
+                act.setIcon(qtlib.getmenuicon(icon))
+            if key:
+                act.setShortcut(key)
+            if tip:
+                act.setStatusTip(tip)
+            if cb:
+                act.triggered.connect(cb)
+            self._actions[name] = act
+            self.addAction(act)
+
+    @pyqtSlot(QPoint)
+    def menuRequest(self, point):
+        actionlist = ['diff', 'edit', 'revert']
+        if not self.contextmenu:
+            menu = QMenu(self)
+            for act in actionlist:
+                menu.addAction(self._actions[act])
+            self.contextmenu = menu
+        self.contextmenu.exec_(self.filelist.viewport().mapToGlobal(point))
+
+    def vdiff(self):
+        filenames = self.getSelectedFiles()
+        if len(filenames) == 0:
+            return
+        opts = {'change':self.ctx.rev()}
+        dlg = visdiff.visualdiff(self.repo.ui, self.repo, filenames, opts)
+        if dlg:
+            dlg.exec_()
+            dlg.deleteLater()
+
+    def revertfile(self):
+        filenames = self.getSelectedFiles()
+        if len(filenames) == 0:
+            return
+        rev = self.ctx.rev()
+        if rev is None:
+            rev = self.ctx.p1().rev()
+        dlg = revert.RevertDialog(self.repo, filenames, rev, self)
+        dlg.exec_()
+        dlg.deleteLater()
+
     def timerEvent(self, event):
         'Periodic poll of currently displayed patch or working file'
-        if not hasattr(self, 'filelistmodel'):
+        if not hasattr(self, 'filelist'):
             return
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if ctx is None:
             return
         if isinstance(ctx, patchctx):
@@ -109,22 +168,29 @@ class ChunksWidget(QWidget):
         ok = True
         repo = self.repo
         ui.pushbuffer()
-        pfiles = {}
-        curdir = os.getcwd()
         try:
             eolmode = ui.config('patch', 'eol', 'strict')
             if eolmode.lower() not in patch.eolmodes:
                 eolmode = 'strict'
             else:
                 eolmode = eolmode.lower()
-            os.chdir(repo.root)
-            if patch.applydiff(ui, fp, pfiles, eolmode=eolmode) < 0:
+            try:
+                # hg-1.9
+                ret = patch.internalpatch(ui, repo, fp, 1, files=None,
+                                          eolmode=eolmode, similarity=0)
+            except TypeError:
+                # hg-1.8
+                pfiles = {}
+                ret = patch.internalpatch(fp, ui, 1, repo.root, pfiles,
+                                          eolmode=eolmode)
+                if updatestate:
+                    cmdutil.updatedir(repo.ui, repo, pfiles)
+            if ret < 0:
                 ok = False
                 self.showMessage.emit(_('Patch failed to apply'))
         except (patch.PatchError, EnvironmentError), err:
             ok = False
             self.showMessage.emit(hglib.tounicode(str(err)))
-        os.chdir(curdir)
         for line in ui.popbuffer().splitlines():
             if line.endswith(wfile + '.rej'):
                 if qtlib.QuestionMsgBox(_('Manually resolve rejected chunks?'),
@@ -136,18 +202,15 @@ class ChunksWidget(QWidget):
                     if dlg.exec_() == QDialog.Accepted:
                         ok = True
                     break
-        if updatestate and ok:
-            # Apply operations specified in git diff headers
-            cmdutil.updatedir(repo.ui, repo, pfiles)
         return ok
 
     def editCurrentFile(self):
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
-            path = ctx._path
+            paths = [ctx._path]
         else:
-            path = self.repo.wjoin(self.currentFile)
-        wctxactions.edit(self, self.repo.ui, self.repo, [path])
+            paths = self.getSelectedFiles()
+        qtlib.editfiles(self.repo, paths, parent=self)
 
     def getSelectedFileAndChunks(self):
         chunks = self.diffbrowse.curchunks
@@ -157,6 +220,9 @@ class ChunksWidget(QWidget):
         else:
             return self.currentFile, []
 
+    def getSelectedFiles(self):
+        return self.filelist.getSelectedFiles()
+
     def deleteSelectedChunks(self):
         'delete currently selected chunks'
         repo = self.repo
@@ -170,7 +236,7 @@ class ChunksWidget(QWidget):
         if not kchunks and qtlib.QuestionMsgBox(_('No chunks remain'),
                                                 _('Remove all file changes?')):
             revertall = True
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
             repo.thgbackup(ctx._path)
             fp = util.atomictempfile(ctx._path, 'wb')
@@ -213,7 +279,7 @@ class ChunksWidget(QWidget):
                 wlock = repo.wlock()
                 try:
                     repo.wopener(self.currentFile, 'wb').write(
-                        repo['.'][self.currentFile].data())
+                        self.diffbrowse.origcontents)
                     fp = cStringIO.StringIO()
                     chunks[0].write(fp)
                     for c in kchunks:
@@ -233,7 +299,7 @@ class ChunksWidget(QWidget):
                     return True
             return False
         repo = self.repo
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
             if wfile in ctx._files:
                 patchchunks = ctx._files[wfile]
@@ -294,11 +360,11 @@ class ChunksWidget(QWidget):
             return False
 
     def getFileList(self):
-        return self.filelistmodel._ctx.files()
+        return self.ctx.files()
 
     def removeFile(self, wfile):
         repo = self.repo
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
             repo.thgbackup(ctx._path)
             fp = util.atomictempfile(ctx._path, 'wb')
@@ -318,14 +384,15 @@ class ChunksWidget(QWidget):
         else:
             repo.thgbackup(repo.wjoin(wfile))
             wasadded = wfile in repo[None].added()
-            commands.revert(repo.ui, repo, repo.wjoin(wfile), no_backup=True)
+            commands.revert(repo.ui, repo, repo.wjoin(wfile), rev='.',
+                            no_backup=True)
             if wasadded:
                 os.unlink(repo.wjoin(wfile))
         self.fileModified.emit()
 
     def getChunksForFile(self, wfile):
         repo = self.repo
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
             if wfile in ctx._files:
                 return ctx._files[wfile]
@@ -346,8 +413,11 @@ class ChunksWidget(QWidget):
             else:
                 return []
 
-    @pyqtSlot(object, object, object)
-    def displayFile(self, file, rev, status):
+    @pyqtSlot(QString, QString)
+    def displayFile(self, file, status):
+        if isinstance(file, (unicode, QString)):
+            file = hglib.fromunicode(file)
+            status = hglib.fromunicode(status)
         if file:
             self.currentFile = file
             path = self.repo.wjoin(file)
@@ -365,7 +435,7 @@ class ChunksWidget(QWidget):
 
     def setContext(self, ctx):
         self.diffbrowse.setContext(ctx)
-        self.filelistmodel.setContext(ctx)
+        self.filelist.setContext(ctx)
         empty = len(ctx.files()) == 0
         self.fileModelEmpty.emit(empty)
         self.fileSelected.emit(not empty)
@@ -374,9 +444,12 @@ class ChunksWidget(QWidget):
             self.diffbrowse.clearDisplay()
             self.diffbrowse.clearChunks()
         self.diffbrowse.updateSummary()
+        self.ctx = ctx
+        for act in ['diff', 'revert']:
+            self._actions[act].setEnabled(ctx.rev() is None)
 
     def refresh(self):
-        ctx = self.filelistmodel._ctx
+        ctx = self.ctx
         if isinstance(ctx, patchctx):
             # if patch mtime has not changed, it could return the same ctx
             ctx = self.repo.changectx(ctx._path)
@@ -384,8 +457,6 @@ class ChunksWidget(QWidget):
             self.repo.thginvalidate()
             ctx = self.repo.changectx(ctx.node())
         self.setContext(ctx)
-        if self.currentFile:
-            self.filelist.selectFile(self.currentFile)
 
     def loadSettings(self, qs, prefix):
         self.diffbrowse.loadSettings(qs, prefix)
@@ -497,7 +568,7 @@ class DiffBrowser(QFrame):
         self.clearDisplay()
 
     def menuRequested(self, point):
-        point = self.sci.mapToGlobal(point)
+        point = self.sci.viewport().mapToGlobal(point)
         return self.sci.createStandardContextMenu().exec_(point)
 
     def loadSettings(self, qs, prefix):
@@ -582,7 +653,7 @@ class DiffBrowser(QFrame):
         self._lastfile = filename
         self.clearChunks()
 
-        fd = fileview.FileData(self._ctx, None, filename, status)
+        fd = filedata.FileData(self._ctx, None, filename, status)
 
         if fd.elabel:
             self.extralabel.setText(fd.elabel)
@@ -623,6 +694,7 @@ class DiffBrowser(QFrame):
                 else:
                     self.sci.markerAdd(start+i, self.vertical)
             start += len(chunk.lines) + 1
+        self.origcontents = fd.olddata
         self.countselected = 0
         self.curchunks = chunks
         for c in chunks[1:]:
