@@ -89,7 +89,7 @@ class CSSLexerSelector(_FilenameLexerSelector):
     _lexer = Qsci.QsciLexerCSS
 
 class XMLLexerSelector(_FilenameLexerSelector):
-    extensions = ('.xhtml', '.xml')
+    extensions = ('.xhtml', '.xml', '.csproj', 'app.config', 'web.config')
     _lexer = Qsci.QsciLexerXML
 
 class HTMLLexerSelector(_FilenameLexerSelector):
@@ -128,6 +128,10 @@ class TeXLexerSelector(_FilenameLexerSelector):
     extensions = ('.tex', '.latex',)
     _lexer = Qsci.QsciLexerTeX
 
+class CSharpLexerSelector(_FilenameLexerSelector):
+    extensions = ('.cs')
+    _lexer = Qsci.QsciLexerCSharp
+
 class DiffLexerSelector(_ScriptLexerSelector):
     extensions = ()
     _lexer = Qsci.QsciLexerDiff
