@@ -51,6 +51,7 @@ class RepoFilterBar(QToolBar):
         self.revsetcombo = combo = QComboBox()
         combo.setEditable(True)
         combo.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        combo.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLength)
         le = combo.lineEdit()
         le.returnPressed.connect(self.returnPressed)
         le.selectionChanged.connect(self.selectionChanged)
@@ -62,8 +63,7 @@ class RepoFilterBar(QToolBar):
         self.clearBtn = QToolButton(self)
         self.clearBtn.setIcon(qtlib.geticon('filedelete'))
         self.clearBtn.setToolTip(_('Clear current query and query text'))
-        self.clearBtn.clicked.connect(le.clear)
-        self.clearBtn.clicked.connect(self.clearRevisionSet)
+        self.clearBtn.clicked.connect(self.onClearButtonClicked)
         self.addWidget(self.clearBtn)
         self.addWidget(combo)
 
@@ -98,6 +98,13 @@ class RepoFilterBar(QToolBar):
         self._initbranchfilter()
         self.refresh()
 
+    def onClearButtonClicked(self):
+        if self.revsetcombo.lineEdit().text():
+            self.revsetcombo.lineEdit().clear()
+        else:
+            self.hide()
+        self.clearRevisionSet.emit()
+
     def setEnableFilter(self, enabled):
         'Enable/disable the changing of the current filter'
         self.revsetcombo.setEnabled(enabled)
@@ -198,6 +205,9 @@ class RepoFilterBar(QToolBar):
             text=_('Branch'), popupMode=QToolButton.InstantPopup,
             statusTip=_('Display graph the named branch only'))
         self._branchMenu = QMenu(self._branchLabel)
+        self._abranchAction = self._branchMenu.addAction(
+            _('Display only active branches'), self.refresh)
+        self._abranchAction.setCheckable(True)
         self._cbranchAction = self._branchMenu.addAction(
             _('Display closed branches'), self.refresh)
         self._cbranchAction.setCheckable(True)
@@ -207,6 +217,8 @@ class RepoFilterBar(QToolBar):
         self._branchLabel.setMenu(self._branchMenu)
 
         self._branchCombo = QComboBox()
+        self._branchCombo.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
+        self._branchCombo.setSizeAdjustPolicy(QComboBox.AdjustToMinimumContentsLength)
         self._branchCombo.currentIndexChanged.connect(self._emitBranchChanged)
         self._branchReloading = False
 
@@ -217,7 +229,11 @@ class RepoFilterBar(QToolBar):
         """Update the list of branches"""
         curbranch = self.branch()
 
-        if self._cbranchAction.isChecked():
+        if self._abranchAction.isChecked():
+            branches = sorted(set([self._repo[n].branch()
+                for n in self._repo.heads()
+                if not self._repo[n].extra().get('close')]))
+        elif self._cbranchAction.isChecked():
             branches = sorted(self._repo.branchtags().keys())
         else:
             branches = self._repo.namedbranches
@@ -227,8 +243,9 @@ class RepoFilterBar(QToolBar):
         self._branchCombo.addItem(self._allBranchesLabel)
         for branch in branches:
             self._branchCombo.addItem(branch)
-        self._branchLabel.setEnabled(self.filterEnabled and len(branches) > 1)
-        self._branchCombo.setEnabled(self.filterEnabled and len(branches) > 1)
+            self._branchCombo.setItemData(self._branchCombo.count() - 1, branch, Qt.ToolTipRole)
+        self._branchLabel.setEnabled(self.filterEnabled and (len(branches) > 1 or self._abranchAction.isChecked()))
+        self._branchCombo.setEnabled(self.filterEnabled and (len(branches) > 1 or self._abranchAction.isChecked()))
         self._branchReloading = False
 
         if not curbranch:
