@@ -5,23 +5,24 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from mercurial import error
-
-from tortoisehg.hgqt import thgrepo, qtlib
-from tortoisehg.util import hglib
+from tortoisehg.util import hglib, paths
 from tortoisehg.hgqt.i18n import _
+from tortoisehg.hgqt import qtlib
 
 from repotreeitem import undumpObject, AllRepoGroupItem, RepoGroupItem
-from repotreeitem import RepoItem, RepoTreeItem
+from repotreeitem import RepoItem, RepoTreeItem, SubrepoItem
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
+import os
+
 
 extractXmlElementName = 'reporegextract'
 reporegistryXmlElementName = 'reporegistry'
 
 repoRegMimeType = 'application/thg-reporegistry'
+repoExternalMimeType = 'text/uri-list'
 
 
 def writeXml(target, item, rootElementName):
@@ -34,7 +35,7 @@ def writeXml(target, item, rootElementName):
     xw.writeEndElement()
     xw.writeEndDocument()
 
-def readXml(source, rootElementName, model):
+def readXml(source, rootElementName):
     itemread = None
     xr = QXmlStreamReader(source)
     if xr.readNextStartElement():
@@ -46,24 +47,39 @@ def readXml(source, rootElementName, model):
     if xr.hasError():
         print str(xr.errorString())
     if xr.readNextStartElement():
-        itemread = undumpObject(xr, model)
+        itemread = undumpObject(xr)
         xr.skipCurrentElement()
     if xr.hasError():
         print str(xr.errorString())
     return itemread
 
-def iterRepoItemFromXml(source, model=None):
+def iterRepoItemFromXml(source):
+    'Used by thgrepo.relatedRepositories to scan the XML file'
     xr = QXmlStreamReader(source)
     while not xr.atEnd():
         t = xr.readNext()
-        if t == QXmlStreamReader.StartElement and xr.name() == 'repo':
-            yield undumpObject(xr, model)
+        if t == QXmlStreamReader.StartElement and xr.name() in ('repo', 'subrepo'):
+            yield undumpObject(xr)
+
+def getRepoItemList(root, includeSubRepos=False):
+    if not includeSubRepos and isinstance(root, RepoItem):
+        return [root]
+    if not isinstance(root, RepoTreeItem):
+        return []
+    return reduce(lambda a, b: a + b,
+                  (getRepoItemList(c, includeSubRepos=includeSubRepos) \
+                    for c in root.childs), [])
+
 
 class RepoTreeModel(QAbstractItemModel):
-    def __init__(self, openrepofunc, filename=None, parent=None):
+
+    def __init__(self, filename, parent, showSubrepos=False,
+            showNetworkSubrepos=False, showShortPaths=False):
         QAbstractItemModel.__init__(self, parent)
 
-        self.openrepofunc = openrepofunc
+        self.showSubrepos = showSubrepos
+        self.showNetworkSubrepos = showNetworkSubrepos
+        self.showShortPaths = showShortPaths
 
         root = None
         all = None
@@ -71,7 +87,7 @@ class RepoTreeModel(QAbstractItemModel):
         if filename:
             f = QFile(filename)
             if f.open(QIODevice.ReadOnly):
-                root = readXml(f, reporegistryXmlElementName, self)
+                root = readXml(f, reporegistryXmlElementName)
                 f.close()
                 if root:
                     for c in root.childs:
@@ -79,6 +95,9 @@ class RepoTreeModel(QAbstractItemModel):
                             all = c
                             break
 
+                    if self.showSubrepos:
+                        self.loadSubrepos(root)
+
         if not root:
             root = RepoTreeItem(self)
             all = AllRepoGroupItem(self)
@@ -86,6 +105,7 @@ class RepoTreeModel(QAbstractItemModel):
 
         self.rootItem = root
         self.allrepos = all
+        self.updateCommonPaths()
 
     # see http://doc.qt.nokia.com/4.6/model-view-model-subclassing.html
 
@@ -131,7 +151,8 @@ class RepoTreeModel(QAbstractItemModel):
     def data(self, index, role):
         if not index.isValid():
             return QVariant()
-        if role not in (Qt.DisplayRole, Qt.EditRole, Qt.DecorationRole):
+        if role not in (Qt.DisplayRole, Qt.EditRole, Qt.DecorationRole,
+                Qt.FontRole):
             return QVariant()
         item = index.internalPointer()
         return item.data(index.column(), role)
@@ -162,7 +183,7 @@ class RepoTreeModel(QAbstractItemModel):
         return res
 
     def mimeTypes(self):
-        return QStringList(repoRegMimeType)
+        return QStringList([repoRegMimeType, repoExternalMimeType])
 
     def mimeData(self, indexes):
         i = indexes[0]
@@ -171,20 +192,37 @@ class RepoTreeModel(QAbstractItemModel):
         writeXml(buf, item, extractXmlElementName)
         d = QMimeData()
         d.setData(repoRegMimeType, buf)
-        d.setUrls([QUrl.fromLocalFile(hglib.tounicode(item.rootpath()))])
+        if isinstance(item, RepoItem):
+            d.setUrls([QUrl.fromLocalFile(hglib.tounicode(item.rootpath()))])
+        else:
+            d.setText(QString(item.name))
         return d
 
     def dropMimeData(self, data, action, row, column, parent):
-        d = str(data.data(repoRegMimeType))
-        itemread = readXml(d, extractXmlElementName, self)
         group = parent.internalPointer()
+        d = str(data.data(repoRegMimeType))
+        if not data.hasUrls():
+            # don't allow nesting of groups
+            row = parent.row()
+            group = self.rootItem
+            parent = QModelIndex()
+        itemread = readXml(d, extractXmlElementName)
+        if itemread is None:
+            return False
         if group is None:
             return False
+        # Avoid copying subrepos multiple times
+        if Qt.CopyAction == action and self.getRepoItem(itemread.rootpath()):
+            return False
         if row < 0:
             row = 0
+        if self.showSubrepos:
+            self.loadSubrepos(itemread)
         self.beginInsertRows(parent, row, row)
         group.insertChild(row, itemread)
         self.endInsertRows()
+        if isinstance(itemread, AllRepoGroupItem):
+            self.allrepos = itemread
         return True
 
     def setData(self, index, value, role):
@@ -195,52 +233,65 @@ class RepoTreeModel(QAbstractItemModel):
             return False
         item = index.internalPointer()
         if item.setData(index.column(), value):
-            self.emit(SIGNAL('dataChanged(index, index)'), index, index)
+            self.dataChanged.emit(index, index)
             return True
         return False
 
     # functions not defined in QAbstractItemModel
 
     def allreposIndex(self):
-        return self.createIndex(0, 0, self.allrepos)
+        return self.createIndex(self.allrepos.row(), 0, self.allrepos)
 
-    def addRepo(self, group, repo):
-        if not repo:
-            caption = _('Select repository directory to add')
-            FD = QFileDialog
-            path = FD.getExistingDirectory(caption=caption,
-                    options=FD.ShowDirsOnly | FD.ReadOnly)
-            if path:
-                try:
-                    lpath = hglib.fromunicode(path)
-                    repo = thgrepo.repository(None, path=lpath)
-                except error.RepoError:
-                    # NOTE: here we cannot pass parent=self because self
-                    # isn't a QWidget. Codes under `if not repo:` should
-                    # be handled by a widget, not by a model.
-                    qtlib.WarningMsgBox(
-                        _('Failed to add repository'),
-                        _('%s is not a valid repository') % path)
-                    return
-            else:
-                return
+    def addRepo(self, group, root, row=-1):
         grp = group
         if grp == None:
             grp = self.allreposIndex()
         rgi = grp.internalPointer()
-        cc = rgi.childCount()
-        self.beginInsertRows(grp, cc, cc + 1)
-        rgi.appendChild(RepoItem(self, repo))
+        if row < 0:
+            row = rgi.childCount()
+
+        # Is the root of the repo that we want to add a subrepo contained
+        # within a repo or subrepo? If so, assume it is an hg subrepo
+        itemIsSubrepo = not paths.find_root(os.path.dirname(root)) is None
+        self.beginInsertRows(grp, row, row)
+        if itemIsSubrepo:
+            ri = SubrepoItem(root)
+        else:
+            ri = RepoItem(root)
+        rgi.insertChild(row, ri)
+
+        if not self.showSubrepos \
+                or (not self.showNetworkSubrepos and paths.netdrive_status(root)):
+            self.endInsertRows()
+            return
+
+        invalidRepoList = ri.appendSubrepos()
+
         self.endInsertRows()
 
-    def getRepoItem(self, reporoot):
-        return self.rootItem.getRepoItem(reporoot)
+        if invalidRepoList:
+            if invalidRepoList[0] == root:
+                qtlib.WarningMsgBox(_('Could not get subrepository list'),
+                    _('It was not possible to get the subrepository list for '
+                    'the repository in:<br><br><i>%s</i>') % root)
+            else:
+                qtlib.WarningMsgBox(_('Could not open some subrepositories'),
+                    _('It was not possible to fully load the subrepository '
+                    'list for the repository in:<br><br><i>%s</i><br><br>'
+                    'The following subrepositories may be missing, broken or '
+                    'on an inconsistent state and cannot be accessed:'
+                    '<br><br><i>%s</i>')  %
+                    (root, "<br>".join(invalidRepoList)))
+
+    def getRepoItem(self, reporoot, lookForSubrepos=False):
+        return self.rootItem.getRepoItem(os.path.normcase(reporoot),
+                    lookForSubrepos=lookForSubrepos)
 
     def addGroup(self, name):
         ri = self.rootItem
         cc = ri.childCount()
         self.beginInsertRows(QModelIndex(), cc, cc + 1)
-        ri.appendChild(RepoGroupItem(self, name, ri))
+        ri.appendChild(RepoGroupItem(name, ri))
         self.endInsertRows()
 
     def write(self, fn):
@@ -248,3 +299,31 @@ class RepoTreeModel(QAbstractItemModel):
         f.open(QIODevice.WriteOnly)
         writeXml(f, self.rootItem, reporegistryXmlElementName)
         f.close()
+
+    def depth(self, index):
+        count = 1
+        while True:
+            index = index.parent()
+            if index.row() < 0:
+                return count
+            count += 1
+
+    def loadSubrepos(self, root, filterFunc=(lambda r: True)):
+        for c in getRepoItemList(root):
+            if filterFunc(c.rootpath()):
+                if self.showNetworkSubrepos \
+                        or not paths.netdrive_status(c.rootpath()):
+                    self.removeRows(0, c.childCount(),
+                        self.createIndex(c.row(), 0, c))
+                    c.appendSubrepos()
+
+    def updateCommonPaths(self, showShortPaths=None):
+        if not showShortPaths is None:
+            self.showShortPaths = showShortPaths
+        for grp in self.rootItem.childs:
+            if isinstance(grp, RepoGroupItem):
+                if self.showShortPaths:
+                    grp.updateCommonPath()
+                else:
+                    grp.updateCommonPath('')
+
