@@ -377,11 +377,14 @@ class RevsetThread(QThread):
         cwd = os.getcwd()
         try:
             os.chdir(self.repo.root)
-            func = revset.match(self.text)
+            func = hglib.revsetmatch(self.repo.ui, self.text)
             l = []
             for c in func(self.repo, range(len(self.repo))):
                 l.append(c)
-            self.showMessage.emit(_('%d matches found') % len(l))
+            if len(l):
+                self.showMessage.emit(_('%d matches found') % len(l))
+            else:
+                self.showMessage.emit(_('No matches found'))
             self.queryIssued.emit(self.query, l)
         except error.ParseError, e:
             if len(e.args) == 2:
@@ -390,6 +393,8 @@ class RevsetThread(QThread):
             else:
                 msg = e.args[0]
             self.showMessage.emit(_('Parse Error: ') + hglib.tounicode(msg))
+        except TypeError:
+            raise
         except Exception, e:
             self.showMessage.emit(_('Invalid query: ')+hglib.tounicode(str(e)))
 
