@@ -14,14 +14,12 @@ import urlparse
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-from mercurial import hg, ui, url, util, error
+from mercurial import hg, ui, url, util, error, demandimport
 from mercurial import merge as mergemod
 
-from tortoisehg.util import hglib, wconfig
+from tortoisehg.util import hglib, wconfig, paths
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, cmdui, thgrepo, rebase, resolve
-
-_schemes = ['local', 'ssh', 'http', 'https']
+from tortoisehg.hgqt import qtlib, cmdui, thgrepo, rebase, resolve, hgrcutil
 
 def parseurl(path):
     if path.startswith('ssh://'):
@@ -40,10 +38,10 @@ def parseurl(path):
         else:
             qtlib.WarningMsgBox(_('Malformed ssh URL'), hglib.tounicode(path))
             host, port, folder = '', '', ''
-    elif path.startswith('http://') or path.startswith('https://'):
+    elif path.startswith(('http://', 'https://', 'svn+https://')):
         snpaqf = urlparse.urlparse(path)
         scheme, netloc, folder, params, query, fragment = snpaqf
-        host, port, user, passwd = url.netlocsplit(netloc)
+        host, port, user, passwd = hglib.netlocsplit(netloc)
         if folder.startswith('/'):
             folder = folder[1:]
     else:
@@ -52,15 +50,18 @@ def parseurl(path):
         scheme = 'local'
     return user, host, port, folder, passwd, scheme
 
-class SyncWidget(QWidget):
+class SyncWidget(QWidget, qtlib.TaskWidget):
     outgoingNodes = pyqtSignal(object)
     incomingBundle = pyqtSignal(QString)
     showMessage = pyqtSignal(unicode)
     pullCompleted = pyqtSignal()
+    pushCompleted = pyqtSignal()
 
     output = pyqtSignal(QString, QString)
     progress = pyqtSignal(QString, object, QString, QString, object)
     makeLogVisible = pyqtSignal(bool)
+    showBusyIcon = pyqtSignal(QString)
+    hideBusyIcon = pyqtSignal(QString)
 
     def __init__(self, repo, parent, **opts):
         QWidget.__init__(self, parent)
@@ -71,6 +72,10 @@ class SyncWidget(QWidget):
         self.setLayout(layout)
         self.setAcceptDrops(True)
 
+        self._schemes = ['local', 'ssh', 'http', 'https']
+        if 'hgsubversion' in repo.extensions():
+            self._schemes.append('svn+https')
+
         self.repo = repo
         self.finishfunc = None
         self.curuser = None
@@ -114,7 +119,7 @@ class SyncWidget(QWidget):
              'hg-outgoing', self.outclicked)
         self.pushAction = \
         newaction(_('Push outgoing changesets to remote repository'),
-             'hg-push', self.pushclicked)
+             'hg-push', lambda: self.pushclicked(True))
         newaction(_('Email outgoing changesets for remote repository'),
              'mail-forward', self.emailclicked)
 
@@ -126,6 +131,9 @@ class SyncWidget(QWidget):
             self.opbuttons.append(a)
             tb.addAction(a)
         tb.addSeparator()
+        newaction(_('Unbundle'),
+             'hg-unbundle', self.unbundle)
+        tb.addSeparator()
         self.stopAction = a = QAction(self)
         a.setToolTip(_('Stop current operation'))
         a.setIcon(qtlib.geticon('process-stop'))
@@ -172,47 +180,52 @@ class SyncWidget(QWidget):
         hbox.setContentsMargins(0, 0, 0, 0)
         layout.addLayout(hbox)
 
+        self.pathEditToolbar = tbar = QToolBar(_('Path Edit Toolbar'))
+        tbar.setIconSize(QSize(16, 16))
+        hbox.addWidget(tbar)
         self.schemecombo = QComboBox()
-        for s in _schemes:
+        for s in self._schemes:
             self.schemecombo.addItem(s)
         self.schemecombo.currentIndexChanged.connect(self.refreshUrl)
-        hbox.addWidget(self.schemecombo)
+        tbar.addWidget(self.schemecombo)
 
-        self.securebutton = QPushButton(qtlib.geticon('thg-password'), '')
-        self.securebutton.setToolTip(
-            _('Manage HTTPS connection security and user authentication'))
-        hbox.addWidget(self.securebutton)
+        a = tbar.addAction(qtlib.geticon('thg-password'), _('Security'))
+        a.setToolTip(_('Manage HTTPS connection security and user authentication'))
+        self.securebutton = a
 
+        fontm = QFontMetrics(self.font())
         self.hostentry = QLineEdit()
         self.hostentry.setToolTip(_('Hostname'))
         self.hostentry.setAcceptDrops(False)
+        self.hostentry.setFixedWidth(30 * fontm.width('9'))
         self.hostentry.textChanged.connect(self.refreshUrl)
-        hbox.addWidget(self.hostentry, 1)
+        tbar.addWidget(self.hostentry)
+
         self.HostAndPortWidgets = [self.hostentry]
         w = QLabel(':')
-        hbox.addWidget(w)
+        tbar.addWidget(w)
         self.HostAndPortWidgets.append(w)
         self.portentry = QLineEdit()
         self.portentry.setAcceptDrops(False)
         self.portentry.setToolTip(_('Port'))
-        fontm = QFontMetrics(self.font())
         self.portentry.setFixedWidth(8 * fontm.width('9'))
         self.portentry.textChanged.connect(self.refreshUrl)
-        hbox.addWidget(self.portentry)
+        tbar.addWidget(self.portentry)
         self.HostAndPortWidgets.append(self.portentry)
         w = QLabel('/')
-        hbox.addWidget(w)
+        tbar.addWidget(w)
         self.HostAndPortWidgets.append(w)
         self.pathentry = QLineEdit()
         self.pathentry.setAcceptDrops(False)
         self.pathentry.setToolTip(_('Path'))
         self.pathentry.textChanged.connect(self.refreshUrl)
-        hbox.addWidget(self.pathentry, 4)
+        tbar.addWidget(self.pathentry)
 
         style = QApplication.style()
-        self.savebutton = QPushButton(style.standardIcon(QStyle.SP_DialogSaveButton), '')
-        self.savebutton.setToolTip(_('Save current URL under an alias'))
-        hbox.addWidget(self.savebutton)
+        a = tbar.addAction(style.standardIcon(QStyle.SP_DialogSaveButton),
+                          _('Save'))
+        a.setToolTip(_('Save current URL under an alias'))
+        self.savebutton = a
 
         hbox = QHBoxLayout()
         hbox.setContentsMargins(0, 0, 0, 0)
@@ -247,8 +260,8 @@ class SyncWidget(QWidget):
 
         self.layout().addLayout(hbox, 1)
 
-        self.savebutton.clicked.connect(self.saveclicked)
-        self.securebutton.clicked.connect(self.secureclicked)
+        self.savebutton.triggered.connect(self.saveclicked)
+        self.securebutton.triggered.connect(self.secureclicked)
         self.postpullbutton.clicked.connect(self.postpullclicked)
         self.optionsbutton.pressed.connect(self.editOptions)
 
@@ -271,6 +284,9 @@ class SyncWidget(QWidget):
             self.setUrl('')
             self.curalias = None
 
+    def canswitch(self):
+        return not self.targetcheckbox.isChecked()
+
     def refreshStatusTips(self):
         url = self.currentUrl(True)
         urlu = hglib.tounicode(url)
@@ -316,13 +332,6 @@ class SyncWidget(QWidget):
             index = 0
         self.targetcombo.setCurrentIndex(index)
 
-    def applyTargetOption(self, cmdline):
-        if self.embedded and self.targetcheckbox.isChecked():
-            idx = self.targetcombo.currentIndex()
-            if idx != -1 and idx < len(self.targetargs):
-                cmdline += self.targetargs[idx]
-        return cmdline
-
     def configChanged(self):
         'Repository is reporting its config files have changed'
         self.reload()
@@ -339,7 +348,7 @@ class SyncWidget(QWidget):
         # Refresh configured paths
         self.paths = {}
         fn = self.repo.join('hgrc')
-        fn, cfg = qtlib.loadIniFile([fn], self)
+        fn, cfg = hgrcutil.loadIniFile([fn], self)
         if 'paths' in cfg:
             for alias in cfg['paths']:
                 self.paths[ alias ] = cfg['paths'][ alias ]
@@ -356,7 +365,7 @@ class SyncWidget(QWidget):
         known.add(os.path.abspath(self.repo.root).lower())
         for path in self.paths.values():
             if hg.islocal(path):
-                known.add(os.path.abspath(hg.localpath(path)).lower())
+                known.add(os.path.abspath(hglib.localpath(path)).lower())
             else:
                 known.add(path)
         related = {}
@@ -371,14 +380,16 @@ class SyncWidget(QWidget):
                 # repositories already opened keep their ui instances in sync
                 repo = thgrepo._repocache[root]
                 ui = repo.ui
-            else:
+            elif paths.is_on_fixed_drive(root):
                 # directly read the repository's configuration file
                 tempui = self.repo.ui.copy()
                 tempui.readconfig(os.path.join(root, '.hg', 'hgrc'))
                 ui = tempui
+            else:
+                continue
             for alias, path in ui.configitems('paths'):
                 if hg.islocal(path):
-                    abs = os.path.abspath(hg.localpath(path)).lower()
+                    abs = os.path.abspath(hglib.localpath(path)).lower()
                 else:
                     abs = path
                 if abs not in known:
@@ -395,8 +406,8 @@ class SyncWidget(QWidget):
         self.urllabel.setText(hglib.tounicode(self.currentUrl(True)))
         schemeIndex = self.schemecombo.currentIndex()
         for w in self.HostAndPortWidgets:
-            w.setHidden(schemeIndex == 0)
-        self.securebutton.setHidden(schemeIndex != 3)
+            w.setDisabled(schemeIndex == 0)
+        self.securebutton.setVisible(schemeIndex >= 3)
 
         opts = []
         for opt, value in self.opts.iteritems():
@@ -409,7 +420,7 @@ class SyncWidget(QWidget):
         self.optionshdrlabel.setVisible(bool(opts))
 
     def currentUrl(self, hidepw):
-        scheme = _schemes[self.schemecombo.currentIndex()]
+        scheme = self._schemes[self.schemecombo.currentIndex()]
         if scheme == 'local':
             return hglib.fromunicode(self.pathentry.text())
         else:
@@ -443,7 +454,7 @@ class SyncWidget(QWidget):
         except TypeError:
             return
         self.updateInProgress = True
-        for i, val in enumerate(_schemes):
+        for i, val in enumerate(self._schemes):
             if scheme == val:
                 self.schemecombo.setCurrentIndex(i)
                 break
@@ -522,21 +533,8 @@ class SyncWidget(QWidget):
             qtlib.InfoMsgBox(_('Repository not local'),
                         _('A terminal shell cannot be opened for remote'))
             return
-        shell = self.repo.shell()
-        if shell:
-            cwd = os.getcwd()
-            try:
-                try:
-                    os.chdir(folder)
-                    QProcess.startDetached(shell)
-                except EnvironmentError, e:
-                    qtlib.InfoMsgBox(_('Repository not found'),
-                                     hglib.tounicode(str(e)))
-            finally:
-                os.chdir(cwd)
-        else:
-            qtlib.InfoMsgBox(_('No shell configured'),
-                        _('A terminal shell must be configured'))
+        qtlib.openshell(folder, 'repo ' + folder)
+
     def removeurl(self):
         if qtlib.QuestionMsgBox(_('Confirm path delete'),
             _('Delete %s from your repo configuration file?') % self.menualias,
@@ -583,11 +581,14 @@ class SyncWidget(QWidget):
         for b in self.opbuttons:
             b.setEnabled(False)
         self.stopAction.setEnabled(True)
-        if not self.embedded:
+        if self.embedded:
+            self.showBusyIcon.emit('thg-sync')
+        else:
             self.cmd.setShowOutput(True)
             self.cmd.setVisible(True)
 
     def commandFinished(self, ret):
+        self.hideBusyIcon.emit('thg-sync')
         self.repo.decrementBusyCount()
         for b in self.opbuttons:
             b.setEnabled(True)
@@ -611,7 +612,13 @@ class SyncWidget(QWidget):
                 cmdline.append(val)
 
         if 'rev' in details and '--rev' not in cmdline:
-            cmdline = self.applyTargetOption(cmdline)
+            if self.embedded and self.targetcheckbox.isChecked():
+                idx = self.targetcombo.currentIndex()
+                if idx != -1 and idx < len(self.targetargs):
+                    args = self.targetargs[idx]
+                    if args[0][2:] not in details:
+                        args[0] = '--rev'
+                    cmdline += args
         if self.opts.get('noproxy'):
             cmdline += ['--config', 'http_proxy.host=']
         if self.opts.get('debug'):
@@ -629,8 +636,8 @@ class SyncWidget(QWidget):
             if self.repo.ui.configbool('insecurehosts', host):
                 cmdline.append('--insecure')
             if user:
-                cleanurl = url.removeauth(cururl)
-                res = url.readauthforuri(self.repo.ui, cleanurl)
+                cleanurl = hglib.removeauth(cururl)
+                res = hglib.readauthforuri(self.repo.ui, cleanurl)
                 if res:
                     group, auth = res
                     if auth.get('username'):
@@ -671,11 +678,11 @@ class SyncWidget(QWidget):
         else:
             self.outclicked()
 
-    def push(self):
+    def push(self, confirm, rev=None, branch=None):
         if self.cmd.core.running():
             self.showMessage.emit(_('sync command already running'))
         else:
-            self.pushclicked()
+            self.pushclicked(confirm, rev, branch)
 
     def pullBundle(self, bundle, rev):
         'accept bundle changesets'
@@ -766,7 +773,7 @@ class SyncWidget(QWidget):
             cmdline += ['--update', '--config', uimerge]
         elif self.cachedpp == 'fetch':
             cmdline[2] = 'fetch'
-        self.run(cmdline, ('force', 'branch', 'rev'))
+        self.run(cmdline, ('force', 'branch', 'rev', 'bookmark'))
 
     def outclicked(self):
         url = self.currentUrl(True)
@@ -841,15 +848,17 @@ class SyncWidget(QWidget):
         self.showMessage.emit(_('Perforce pending...'))
         self.run(['--repository', self.repo.root, 'p4pending', '--verbose'], ())
 
-    def pushclicked(self):
+    def pushclicked(self, confirm, rev=None, branch=None):
         url = self.currentUrl(True)
         urlu = hglib.tounicode(url)
-        if not hg.islocal(self.currentUrl(False)):
+        if (not hg.islocal(self.currentUrl(False)) and confirm
+            and not self.targetcheckbox.isChecked()):
             r = qtlib.QuestionMsgBox(_('Confirm Push to remote Repository'),
                                      _('Push to remote repository\n%s\n?')
                                      % urlu)
             if not r:
                 self.showMessage.emit(_('Push to %s aborted') % urlu)
+                self.pushCompleted.emit()
                 return
 
         self.showMessage.emit(_('Pushing to %s...') % urlu)
@@ -858,9 +867,14 @@ class SyncWidget(QWidget):
                 self.showMessage.emit(_('Push to %s completed') % urlu)
             else:
                 self.showMessage.emit(_('Push to %s aborted, ret %d') % (urlu, ret))
+            self.pushCompleted.emit()
         self.finishfunc = finished
         cmdline = ['--repository', self.repo.root, 'push']
-        self.run(cmdline, ('force', 'new-branch', 'branch', 'rev'))
+        if rev:
+            cmdline.extend(['--rev', str(rev)])
+        if branch:
+            cmdline.extend(['--branch', branch])
+        self.run(cmdline, ('force', 'new-branch', 'branch', 'rev', 'bookmark'))
 
     def postpullclicked(self):
         dlg = PostPullDialog(self.repo, self)
@@ -891,11 +905,26 @@ class SyncWidget(QWidget):
                     '--template', '{node}\n']
         self.run(cmdline, ('force', 'branch', 'rev'))
 
+    def unbundle(self):
+        caption = _("Select bundle file")
+        _FILE_FILTER = "%s" % _("Bundle files (*.hg)")
+        bundlefile = QFileDialog.getOpenFileName(parent=self, caption=caption,
+                                    directory=self.repo.root,
+                                    filter=_FILE_FILTER)
+        if bundlefile:
+            # Select the "Local" scheme
+            self.schemecombo.setCurrentIndex(0)
+            # Set the pull source to the selected bundle file
+            self.pathentry.setText(bundlefile)
+            # Execute the incomming command, which will show the revisions in
+            # the bundle, and let the user accept or reject them
+            self.inclicked()
+
     @pyqtSlot(QString)
     def removeAlias(self, alias):
         alias = hglib.fromunicode(alias)
         fn = self.repo.join('hgrc')
-        fn, cfg = qtlib.loadIniFile([fn], self)
+        fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to remove URL'),
                    _('Iniparse must be installed.'), parent=self)
@@ -993,7 +1022,7 @@ class PostPullDialog(QDialog):
 
     def accept(self):
         path = self.repo.join('hgrc')
-        fn, cfg = qtlib.loadIniFile([path], self)
+        fn, cfg = hgrcutil.loadIniFile([path], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save post pull operation'),
                    _('Iniparse must be installed.'), parent=self)
@@ -1036,7 +1065,7 @@ class SaveDialog(QDialog):
 
         user, host, port, folder, passwd, scheme = parseurl(origurl)
         if (user or passwd) and scheme in ('http', 'https'):
-            cleanurl = url.removeauth(origurl)
+            cleanurl = hglib.removeauth(origurl)
             def showurl(showclean):
                 newurl = showclean and cleanurl or safeurl
                 self.urllabel.setText(hglib.tounicode(newurl))
@@ -1063,7 +1092,7 @@ class SaveDialog(QDialog):
 
     def accept(self):
         fn = self.repo.join('hgrc')
-        fn, cfg = qtlib.loadIniFile([fn], self)
+        fn, cfg = hgrcutil.loadIniFile([fn], self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save an URL'),
                    _('Iniparse must be installed.'), parent=self)
@@ -1119,14 +1148,18 @@ class SecureDialog(QDialog):
                             ~Qt.WindowContextHelpButtonHint)
 
         # if the already user has an [auth] configuration for this URL, use it
-        cleanurl = url.removeauth(origurl)
-        res = url.readauthforuri(repo.ui, cleanurl)
+        cleanurl = hglib.removeauth(origurl)
+        res = hglib.readauthforuri(repo.ui, cleanurl)
         if res:
             self.alias, auth = res
         else:
             self.alias, auth = host, {}
         self.repo = repo
         self.host = host
+        if cleanurl.startswith('svn+https://'):
+            self.schemes = 'svn+https'
+        else:
+            self.schemes = None
 
         self.setLayout(QVBoxLayout())
         self.layout().addWidget(QLabel(_('<b>Host:</b> %s') % uhost))
@@ -1226,8 +1259,8 @@ are expanded in the filename.'''))
         qtlib.openhelpcontents('sync.html#security')
 
     def accept(self):
-        path = util.user_rcpath()
-        fn, cfg = qtlib.loadIniFile(path, self)
+        path = hglib.user_rcpath()
+        fn, cfg = hgrcutil.loadIniFile(path, self)
         if not hasattr(cfg, 'write'):
             qtlib.WarningMsgBox(_('Unable to save authentication'),
                    _('Iniparse must be installed.'), parent=self)
@@ -1263,6 +1296,7 @@ are expanded in the filename.'''))
         setorclear('auth', self.alias+'.password', password)
         setorclear('auth', self.alias+'.key', key)
         setorclear('auth', self.alias+'.cert', chain)
+        setorclear('auth', self.alias+'.schemes', self.schemes)
 
         self.repo.incrementBusyCount()
         try:
@@ -1349,7 +1383,7 @@ class PathsModel(QAbstractTableModel):
         self.headers = (_('Alias'), _('URL'))
         self.rows = []
         for alias, path in pathlist:
-            safepath = url.hidepassword(path)
+            safepath = hglib.hidepassword(path)
             ualias = hglib.tounicode(alias)
             usafepath = hglib.tounicode(safepath)
             self.rows.append([ualias, usafepath, path])
@@ -1452,6 +1486,5 @@ class OptionsDialog(QDialog):
 
 
 def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
     repo = thgrepo.repository(ui, path=paths.find_root())
     return SyncWidget(repo, None, **opts)
