@@ -71,7 +71,7 @@ class ImportDialog(QDialog):
         grid.addWidget(self.p0chk, 2, 1, Qt.AlignLeft)
 
         ### patch list
-        self.cslist = cslist.ChangesetList()
+        self.cslist = cslist.ChangesetList(self.repo)
         self.cslistrow = cslistrow = 4
         self.cslistcol = cslistcol = 1
         grid.addWidget(self.cslist, cslistrow, cslistcol,
@@ -145,7 +145,7 @@ class ImportDialog(QDialog):
 
     def checkStatus(self):
         def activated():
-            dlg = commit.CommitDialog([], dict(root=self.repo.root), self)
+            dlg = commit.CommitDialog(self.repo, [], {}, self)
             dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
             self.checkStatus()
@@ -178,9 +178,8 @@ class ImportDialog(QDialog):
                                                 filter=_FILE_FILTER)
         if filelist:
             # Qt file browser uses '/' in paths, even on Windows.
-            filelist = [str(x.replace('/', os.sep)) for x in filelist]
-            response = os.pathsep.join(filelist)
-            self.src_combo.setEditText(response)
+            nl = QStringList([QDir.toNativeSeparators(x) for x in filelist])
+            self.src_combo.setEditText(nl.join(os.pathsep))
             self.src_combo.setFocus()
 
     def browsedir(self):
@@ -189,8 +188,7 @@ class ImportDialog(QDialog):
                                                 directory=self.repo.root,
                                                 caption=caption)
         if path:
-            response = str(path.replace('/', os.sep))
-            self.src_combo.setEditText(response)
+            self.src_combo.setEditText(QDir.toNativeSeparators(path))
             self.src_combo.setFocus()
 
     def getcliptext(self):
@@ -223,8 +221,7 @@ class ImportDialog(QDialog):
             self.cslist.clear()
             self.import_btn.setDisabled(True)
         else:
-            self.cslist.update(self.repo,
-                               [os.path.abspath(p) for p in patches])
+            self.cslist.update([os.path.abspath(p) for p in patches])
             self.import_btn.setEnabled(True)
         self.updatestatus()
 
@@ -253,6 +250,8 @@ class ImportDialog(QDialog):
             os.pathsep.join(hglib.tounicode(p) for p in paths))
 
     def thgimport(self):
+        if self.cslist.curitems is None:
+            return
         idx = self.targetcombo.currentIndex()
         if idx == 1:
             # import to shelf
