@@ -40,16 +40,16 @@ def repository(_ui=None, path='', create=False, bundle=None):
         if _ui is None:
             _ui = uimod.ui()
         repo = bundlerepo.bundlerepository(_ui, path, bundle)
-        repo._pyqtobj = ThgRepoWrapper(repo)
         repo.__class__ = _extendrepo(repo)
+        repo._pyqtobj = ThgRepoWrapper(repo)
         return repo
     if create or path not in _repocache:
         if _ui is None:
             _ui = uimod.ui()
         try:
             repo = hg.repository(_ui, path, create)
-            repo._pyqtobj = ThgRepoWrapper(repo)
             repo.__class__ = _extendrepo(repo)
+            repo._pyqtobj = ThgRepoWrapper(repo)
             _repocache[path] = repo
             return repo
         except EnvironmentError:
@@ -65,6 +65,7 @@ class ThgRepoWrapper(QObject):
     configChanged = pyqtSignal()
     repositoryChanged = pyqtSignal()
     repositoryDestroyed = pyqtSignal()
+    workingDirectoryChanged = pyqtSignal()
     workingBranchChanged = pyqtSignal()
 
     def __init__(self, repo):
@@ -73,35 +74,64 @@ class ThgRepoWrapper(QObject):
         self.busycount = 0
         repo.configChanged = self.configChanged
         repo.repositoryChanged = self.repositoryChanged
-        repo.workingBranchChanged = self.workingBranchChanged
         repo.repositoryDestroyed = self.repositoryDestroyed
+        repo.workingDirectoryChanged = self.workingDirectoryChanged
+        repo.workingBranchChanged = self.workingBranchChanged
         self.recordState()
-        try:
-            freq = repo.ui.config('tortoisehg', 'pollfreq', '500')
-            freq = max(100, int(freq))
-        except:
-            freq = 500
         if isinstance(repo, bundlerepo.bundlerepository):
-            dbgoutput('not starting timer for bundle repository')
+            dbgoutput('not watching F/S events for bundle repository')
         else:
-            self._timerevent = self.startTimer(freq)
+            self.watcher = QFileSystemWatcher(self)
+            self.watcher.addPath(repo.path)
+            self.watcher.directoryChanged.connect(self.onDirChange)
+            self.watcher.fileChanged.connect(self.onFileChange)
+            self.addMissingPaths()
 
-    def timerEvent(self, event):
+    @pyqtSlot(QString)
+    def onDirChange(self, directory):
+        'Catch any writes to .hg/ folder, most importantly lock files'
+        self.pollStatus()
+        self.addMissingPaths()
+
+    @pyqtSlot(QString)
+    def onFileChange(self, file):
+        'Catch writes or deletions of files we are interested in'
+        self.pollStatus()
+        self.addMissingPaths()
+
+    def addMissingPaths(self):
+        'Add files to watcher that may have been added or replaced'
+        existing = [f for f in self._getwatchedfiles() if os.path.isfile(f)]
+        files = [unicode(f) for f in self.watcher.files()]
+        for f in existing:
+            if hglib.tounicode(f) not in files:
+                dbgoutput('add file to watcher:', f)
+                self.watcher.addPath(f)
+        _, files = self.repo.uifiles()
+        for f in files:
+            if f and os.path.exists(f) and hglib.tounicode(f) not in files:
+                dbgoutput('add ui file to watcher:', f)
+                self.watcher.addPath(f)
+
+    def pollStatus(self):
         if not os.path.exists(self.repo.path):
             dbgoutput('Repository destroyed', self.repo.root)
             self.repositoryDestroyed.emit()
-            self.killTimer(self._timerevent)
+            # disable watcher by removing all watched paths
+            dirs = self.watcher.directories()
+            if dirs:
+                self.watcher.removePaths(dirs)
+            files = self.watcher.files()
+            if files:
+                self.watcher.removePaths(files)
             if self.repo.root in _repocache:
                 del _repocache[self.repo.root]
-        elif self.busycount == 0:
-            self.pollStatus()
-        else:
-            dbgoutput('no poll, busy', self.busycount)
-
-    def pollStatus(self):
-        if not os.path.exists(self.repo.path) or self.locked():
+            return
+        if self.locked():
+            dbgoutput('locked, aborting')
             return
         if self._checkdirstate():
+            dbgoutput('dirstate changed, exiting')
             return
         self._checkrepotime()
         self._checkuimtime()
@@ -131,8 +161,7 @@ class ThgRepoWrapper(QObject):
         except EnvironmentError:
             return None
 
-    def _getrepomtime(self):
-        'Return the last modification time for the repo'
+    def _getwatchedfiles(self):
         watchedfiles = [self.repo.sjoin('00changelog.i')]
         watchedfiles.append(self.repo.join('localtags'))
         watchedfiles.append(self.repo.join('bookmarks'))
@@ -141,9 +170,13 @@ class ThgRepoWrapper(QObject):
             watchedfiles.append(self.repo.mq.join('series'))
             watchedfiles.append(self.repo.mq.join('guards'))
             watchedfiles.append(self.repo.join('patches.queue'))
+        return watchedfiles
+
+    def _getrepomtime(self):
+        'Return the last modification time for the repo'
         try:
-            mtime = [os.path.getmtime(wf) for wf in watchedfiles \
-                     if os.path.isfile(wf)]
+            existing = [f for f in self._getwatchedfiles() if os.path.isfile(f)]
+            mtime = [os.path.getmtime(wf) for wf in existing]
             if mtime:
                 return max(mtime)
         except EnvironmentError:
@@ -213,7 +246,7 @@ class ThgRepoWrapper(QObject):
         except (EnvironmentError, ValueError):
             pass
 
-_uiprops = '''_uifiles _uimtime _shell postpull tabwidth maxdiff
+_uiprops = '''_uifiles _uimtime postpull tabwidth maxdiff
               deadbranches _exts _thghiddentags displayname summarylen
               shortname mergetools namedbranches'''.split()
 
@@ -273,7 +306,7 @@ def _extendrepo(repo):
             if MQ not in use.'''
             if not hasattr(self, 'mq'): return []
 
-            self.mq.parse_series()
+            self.mq.parseseries()
             return self.mq.series[:]
 
         @property
@@ -287,18 +320,6 @@ def _extendrepo(repo):
             else:
                 return n
 
-        @propertycache
-        def _shell(self):
-            s = self.ui.config('tortoisehg', 'shell')
-            if s:
-                return s
-            if sys.platform == 'darwin':
-                return None # Terminal.App does not support open-to-folder
-            elif os.name == 'nt':
-                return 'cmd.exe'
-            else:
-                return 'xterm'
-
         @propertycache
         def _uifiles(self):
             cfg = self.ui._ucfg
@@ -423,10 +444,6 @@ def _extendrepo(repo):
                 heads.extend(nodes)
             return heads
 
-        def shell(self):
-            'Returns terminal shell configured for this repo'
-            return self._shell
-
         def uifiles(self):
             'Returns latest mtime and complete list of config files'
             return self._uimtime, self._uifiles
@@ -451,9 +468,20 @@ def _extendrepo(repo):
         def thgshelves(self):
             self.shelfdir = sdir = self.join('shelves')
             if os.path.isdir(sdir):
-                return os.listdir(sdir)
+                def getModificationTime(x):
+                    return os.path.getmtime(os.path.join(sdir, x))
+                shelves = sorted(os.listdir(sdir),
+                    key=getModificationTime, reverse=True)
+                return [s for s in shelves if \
+                           os.path.isfile(os.path.join(self.shelfdir, s))]
             return []
 
+        def makeshelf(self, patch):
+            if not os.path.exists(self.shelfdir):
+                os.mkdir(self.shelfdir)
+            f = open(os.path.join(self.shelfdir, patch), "wb")
+            f.close()
+
         def thginvalidate(self):
             'Should be called when mtime of repo store/dirstate are changed'
             self.dirstate.invalidate()
