@@ -10,12 +10,11 @@ Main Qt4 application for TortoiseHg
 
 import os
 import sys
-
+from mercurial import ui
 from mercurial.error import RepoError
-
 from tortoisehg.util import paths, hglib
 
-from tortoisehg.hgqt import repomodel, thgrepo, cmdui, qtlib
+from tortoisehg.hgqt import thgrepo, cmdui, qtlib, mq
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt.repowidget import RepoWidget
 from tortoisehg.hgqt.reporegistry import RepoRegistryView
@@ -28,26 +27,36 @@ from PyQt4.QtGui import *
 
 class ThgTabBar(QTabBar):
     def mouseReleaseEvent(self, event):
-        
+
         if event.button() == Qt.MidButton:
             self.tabCloseRequested.emit(self.tabAt(event.pos()))
-            
+
         super(QTabBar, self).mouseReleaseEvent(event)
 
 class Workbench(QMainWindow):
     """hg repository viewer/browser application"""
     finished = pyqtSignal(int)
+    activeRepoChanged = pyqtSignal(QString)
 
     def __init__(self):
         QMainWindow.__init__(self)
+        self.ui = ui.ui()
 
         self.setupUi()
         self.setWindowTitle(_('TortoiseHg Workbench'))
-
         self.reporegistry = rr = RepoRegistryView(self)
         rr.setObjectName('RepoRegistryView')
+        rr.showMessage.connect(self.showMessage)
+        rr.openRepo.connect(self.openRepo)
         rr.hide()
         self.addDockWidget(Qt.LeftDockWidgetArea, rr)
+        self.activeRepoChanged.connect(rr.setActiveTabRepo)
+
+        self.mqpatches = p = mq.MQPatchesWidget(self)
+        p.setObjectName('MQPatchesWidget')
+        p.showMessage.connect(self.showMessage)
+        p.hide()
+        self.addDockWidget(Qt.LeftDockWidgetArea, p)
 
         self.log = LogDockWidget(self)
         self.log.setObjectName('Log')
@@ -57,10 +66,8 @@ class Workbench(QMainWindow):
 
         self._setupActions()
 
-        rr.openRepoSignal.connect(self.openRepo)
-
-        self.repoTabChanged()
         self.restoreSettings()
+        self.repoTabChanged()
         self.setAcceptDrops(True)
         if os.name == 'nt':
             # Allow CTRL+Q to close Workbench on Windows
@@ -75,6 +82,10 @@ class Workbench(QMainWindow):
                                     self.openRepository)
             qt_mac_set_dock_menu(self.dockMenu)
 
+        # Create the actions that will be displayed on the context menu
+        self.createActions()
+        self.lastClosedRepoRootList = []
+        
     def setupUi(self):
         desktopgeom = qApp.desktop().availableGeometry()
         self.resize(desktopgeom.size() * 0.8)
@@ -87,6 +98,10 @@ class Workbench(QMainWindow):
         tw.setTabsClosable(True)
         tw.setMovable(True)
         tw.tabBar().hide()
+        tw.tabBar().setContextMenuPolicy(Qt.CustomContextMenu)
+        tw.tabBar().customContextMenuRequested.connect(self.tabBarContextMenuRequest)
+        tw.lastClickedTab = -1 # No tab clicked yet
+
         sp = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
         sp.setHorizontalStretch(1)
         sp.setVerticalStretch(1)
@@ -106,6 +121,7 @@ class Workbench(QMainWindow):
 
         self.menuFile = self.menubar.addMenu(_("&File"))
         self.menuView = self.menubar.addMenu(_("&View"))
+        self.menuViewregistryopts = QMenu(_('Workbench Toolbars'), self)
         self.menuRepository = self.menubar.addMenu(_("&Repository"))
         self.menuHelp = self.menubar.addMenu(_("&Help"))
 
@@ -182,17 +198,21 @@ class Workbench(QMainWindow):
             if toolbar:
                 getattr(self, '%stbar' % toolbar).addSeparator()
 
-        newaction(_("&Open Repository..."), self.openRepository,
-                  shortcut='Open', menu='file')
-        newaction(_("&Close Repository"), self.closeRepository,
-                  shortcut='Close', enabled='repoopen', menu='file')
-        newseparator(menu='file')
         newaction(_("&New Repository..."), self.newRepository,
                   shortcut='New', menu='file', icon='hg-init')
         newaction(_("Clone Repository..."), self.cloneRepository,
                   shortcut=modifiedkeysequence('New', modifier='Shift'),
                   menu='file', icon='hg-clone')
         newseparator(menu='file')
+        newaction(_("&Open Repository..."), self.openRepository,
+                  shortcut='Open', menu='file')
+        closerepo = newaction(_("&Close Repository"), self.closeRepository,
+                  shortcut='Close', enabled='repoopen', menu='file')
+        if os.name == 'nt':
+            sc = closerepo.shortcuts()
+            sc.append(keysequence('Ctrl+W'))
+            closerepo.setShortcuts(sc)
+        newseparator(menu='file')
         newaction(_('&Settings...'), self.editSettings, icon='settings_user',
                   shortcut='Preferences', menu='file')
         newseparator(menu='file')
@@ -205,9 +225,11 @@ class Workbench(QMainWindow):
         self.docktbar.addAction(a)
         self.menuView.addAction(a)
 
-        self.actionShowPaths = \
-        newaction(_("Show Paths"), self.reporegistry.showPaths,
-                  checkable=True, menu='view')
+        a = self.mqpatches.toggleViewAction()
+        a.setText(_('Show Patch Queue'))
+        a.setIcon(qtlib.geticon('thg-mq'))
+        self.docktbar.addAction(a)
+        self.menuView.addAction(a)
 
         a = self.log.toggleViewAction()
         a.setText(_('Show Output &Log'))
@@ -216,6 +238,27 @@ class Workbench(QMainWindow):
         self.docktbar.addAction(a)
         self.menuView.addAction(a)
 
+        newseparator(menu='view')
+        self.menuViewregistryopts = self.menuView.addMenu(_('Repository Registry Options'))
+        self.actionShowPaths = \
+        newaction(_("Show Paths"), self.reporegistry.showPaths,
+                  checkable=True, menu='viewregistryopts')
+
+        self.actionShowSubrepos = \
+            newaction(_("Show Subrepos on Registry"),
+                self.reporegistry.setShowSubrepos,
+                  checkable=True, menu='viewregistryopts')
+
+        self.actionShowNetworkSubrepos = \
+            newaction(_("Show Subrepos for remote repositories"),
+                self.reporegistry.setShowNetworkSubrepos,
+                  checkable=True, menu='viewregistryopts')
+
+        self.actionShowShortPaths = \
+            newaction(_("Show Short Paths"),
+                self.reporegistry.setShowShortPaths,
+                  checkable=True, menu='viewregistryopts')
+
         newseparator(menu='view')
         newaction(_("Choose Log Columns..."), self.setHistoryColumns,
                   menu='view')
@@ -226,23 +269,20 @@ class Workbench(QMainWindow):
 
         self.actionGroupTaskView = QActionGroup(self)
         self.actionGroupTaskView.triggered.connect(self.onSwitchRepoTaskTab)
-        def addtaskview(icon, label, data=None):
-            if data is None:
-                data = len(self.actionGroupTaskView.actions())
-            a = newaction(label, icon=None, checkable=True, data=data,
+        def addtaskview(icon, label, name):
+            a = newaction(label, icon=None, checkable=True, data=name,
                           enabled='repoopen', menu='view')
             a.setIcon(qtlib.geticon(icon))
             self.actionGroupTaskView.addAction(a)
             self.tasktbar.addAction(a)
             return a
-        # NOTE: Sequence must match that in repowidget.py
-        addtaskview('hg-log', _("Revision &Details"))
-        addtaskview('hg-commit', _('&Commit'))
-        addtaskview('thg-sync', _('S&ynchronize'))
-        addtaskview('hg-annotate', _('&Manifest'))
-        addtaskview('hg-grep', _('&Search'))
+        addtaskview('hg-log', _("Revision &Details"), 'log')
+        addtaskview('hg-commit', _('&Commit'), 'commit')
         self.actionSelectTaskMQ = \
-                addtaskview('thg-mq', _('Patch &Queue'), 'mq')
+                addtaskview('thg-qrefresh', _('MQ Patch'), 'mq')
+        addtaskview('thg-sync', _('S&ynchronize'), 'sync')
+        addtaskview('hg-annotate', _('&Manifest'), 'manifest')
+        addtaskview('hg-grep', _('&Search'), 'grep')
         self.actionSelectTaskPbranch = \
                 addtaskview('branch', _('&Patch Branch'), 'pbranch')
         newseparator(menu='view')
@@ -293,7 +333,9 @@ class Workbench(QMainWindow):
                   menu='repository')
 
         newaction(_("Help"), self.onHelp, menu='help', icon='help-browser')
-        newaction(_("About"), self.onAbout, menu='help', icon='thg-logo')
+        newaction(_("About Qt"), QApplication.aboutQt, menu='help')
+        newaction(_("About TortoiseHg"), self.onAbout, menu='help',
+                  icon='thg-logo')
 
         newseparator(toolbar='edit')
         self.actionBack = \
@@ -332,30 +374,122 @@ class Workbench(QMainWindow):
 
         self.updateMenu()
 
-    @pyqtSlot(QAction)
+    def _action_defs(self):
+        a = [("closetab", _("Close tab"), '',
+                _("Close tab"), self.closeLastClickedTab),
+             ("closeothertabs", _("Close other tabs"), '',
+                _("Close other tabs"), self.closeNotLastClickedTabs),
+             ("reopenlastclosed", _("Undo close tab"), '',
+                _("Reopen last closed tab"), self.reopenLastClosedTabs),
+             ("reopenlastclosedgroup", _("Undo close other tabs"), '',
+                _("Reopen last closed tab group"), self.reopenLastClosedTabs),
+             ]
+        return a
+
+    def createActions(self):
+        self._actions = {}
+        for name, desc, icon, tip, cb in self._action_defs():
+            self._actions[name] = QAction(desc, self)
+        QTimer.singleShot(0, self.configureActions)
+
+    def configureActions(self):
+        for name, desc, icon, tip, cb in self._action_defs():
+            act = self._actions[name]
+            if icon:
+                act.setIcon(qtlib.getmenuicon(icon))
+            if tip:
+                act.setStatusTip(tip)
+            if cb:
+                act.triggered.connect(cb)
+            self.addAction(act)
+
+    @pyqtSlot(QPoint)
+    def tabBarContextMenuRequest(self, point):
+        # Activate the clicked tab
+        clickedwidget = qApp.widgetAt(self.repoTabsWidget.mapToGlobal(point))
+        if not clickedwidget or \
+            not isinstance(clickedwidget, ThgTabBar):
+            return
+        self.repoTabsWidget.lastClickedTab = -1
+
+        clickedtabindex = clickedwidget.tabAt(point)
+        if clickedtabindex > -1:
+            self.repoTabsWidget.lastClickedTab = clickedtabindex
+        else:
+            self.repoTabsWidget.lastClickedTab = self.repoTabsWidget.currentIndex()
+
+        actionlist = ['closetab', 'closeothertabs']
+
+        existingClosedRepoList = []
+
+        for reporoot in self.lastClosedRepoRootList:
+            if os.path.isdir(reporoot):
+                existingClosedRepoList.append(reporoot)
+        self.lastClosedRepoRootList = existingClosedRepoList
+
+        if len(self.lastClosedRepoRootList) > 1:
+            actionlist += ['', 'reopenlastclosedgroup']
+        elif len(self.lastClosedRepoRootList) > 0:
+            actionlist += ['', 'reopenlastclosed']
+
+        contextmenu = QMenu(self)
+        for act in actionlist:
+            if act:
+                contextmenu.addAction(self._actions[act])
+            else:
+                contextmenu.addSeparator()
+
+        if actionlist:
+            contextmenu.exec_(self.repoTabsWidget.mapToGlobal(point))
+
+    def closeLastClickedTab(self):
+        if self.repoTabsWidget.lastClickedTab > -1:
+            self.repoTabCloseRequested(self.repoTabsWidget.lastClickedTab)
+
+    def _closeOtherTabs(self, tabIndex):
+        if tabIndex > -1:
+            tb = self.repoTabsWidget.tabBar()
+            tb.setCurrentIndex(tabIndex)
+            closedRepoRootList = []
+            for idx in range(tb.count()-1, -1, -1):
+                if idx != tabIndex:
+                    self.repoTabCloseRequested(idx)
+                    # repoTabCloseRequested updates self.lastClosedRepoRootList
+                    closedRepoRootList += self.lastClosedRepoRootList
+            self.lastClosedRepoRootList = closedRepoRootList
+
+
+    def closeNotLastClickedTabs(self):
+        self._closeOtherTabs(self.repoTabsWidget.lastClickedTab)
+
     def onSwitchRepoTaskTab(self, action):
         rw = self.repoTabsWidget.currentWidget()
-        if not rw: return
-        index, wasint = action.data().toInt()
-        if wasint:
-            rw.taskTabsWidget.setCurrentIndex(index)
-        else:
+        if rw:
             rw.switchToNamedTaskTab(str(action.data().toString()))
 
-    def openRepo(self, repopath, reuse=False):
-        """ Open repo by openRepoSignal from reporegistry """
-        if isinstance(repopath, (unicode, QString)):  # as Qt slot
-            repopath = hglib.fromunicode(repopath)
-        self._openRepo(path=repopath, reuse=reuse)
+    @pyqtSlot(QString, bool)
+    def openRepo(self, root, reuse):
+        """ Open repo by openRepoSignal from reporegistry [unicode] """
+        root = hglib.fromunicode(root)
+        self._openRepo(root, reuse)
 
-    @pyqtSlot(unicode)
-    def showRepo(self, path):
+    @pyqtSlot(QString)
+    def openLinkedRepo(self, path):
+        self.showRepo(path)
+        rw = self.repoTabsWidget.currentWidget()
+        if rw:
+            rw.taskTabsWidget.setCurrentIndex(rw.commitTabIndex)
+
+    @pyqtSlot(QString)
+    def showRepo(self, root):
         """Activate the repo tab or open it if not available [unicode]"""
+        root = hglib.fromunicode(root)
         for i in xrange(self.repoTabsWidget.count()):
             w = self.repoTabsWidget.widget(i)
-            if hglib.tounicode(w.repo.root) == path:
-                return self.repoTabsWidget.setCurrentIndex(i)
-        self.openRepo(path)
+            if hglib.tounicode(w.repo.root) == os.path.normpath(root):
+                self.repoTabsWidget.setCurrentIndex(i)
+                return
+        self._openRepo(root, False)
 
     @pyqtSlot(unicode, QString)
     def setRevsetFilter(self, path, filter):
@@ -404,8 +538,9 @@ class Workbench(QMainWindow):
         self.updateToolBarActions()
         tw = self.repoTabsWidget
         w = tw.currentWidget()
-
-        if tw.count() < 2:
+        if ((tw.count() == 0) or
+            ((tw.count() == 1) and
+             not self.ui.configbool('tortoisehg', 'forcerepotab', False))):
             tw.tabBar().hide()
         else:
             tw.tabBar().show()
@@ -423,7 +558,7 @@ class Workbench(QMainWindow):
         if w:
             self.filtertbaction.setChecked(w.filterBarVisible())
 
-    def updateTaskViewMenu(self, taskIndex=0):
+    def updateTaskViewMenu(self):
         'Update task tab menu for current repository'
         if self.repoTabsWidget.count() == 0:
             for a in self.actionGroupTaskView.actions():
@@ -436,12 +571,12 @@ class Workbench(QMainWindow):
             self.actionSelectTaskMQ.setVisible('mq' in exts)
             self.actionSelectTaskPbranch.setVisible('pbranch' in exts)
             taskIndex = repoWidget.taskTabsWidget.currentIndex()
-            if taskIndex <= 4: # count of standard task tabs
-                self.actionGroupTaskView.actions()[taskIndex].setChecked(True)
-            elif taskIndex == repoWidget.namedTabs.get('mq', None):
-                self.actionSelectTaskMQ.setChecked(True)
-            elif taskIndex == repoWidget.namedTabs.get('pbranch', None):
-                self.actionSelectTaskPbranch.setChecked(True)
+            for name, idx in repoWidget.namedTabs.iteritems():
+                if idx == taskIndex:
+                    break
+            for action in self.actionGroupTaskView.actions():
+                if str(action.data().toString()) == name:
+                    action.setChecked(True)
 
     @pyqtSlot()
     def updateHistoryActions(self):
@@ -456,24 +591,49 @@ class Workbench(QMainWindow):
         self.repoTabsWidget.setCurrentWidget(widget)
         index = self.repoTabsWidget.currentIndex()
         if widget.closeRepoWidget():
+            w = self.repoTabsWidget.widget(index)
+            try:
+                reporoot = w.repo.root
+            except:
+                reporoot = ''
             self.repoTabsWidget.removeTab(index)
             widget.deleteLater()
             self.updateMenu()
+            self.lastClosedRepoRootList = [reporoot]
 
     def repoTabCloseRequested(self, index):
         tw = self.repoTabsWidget
-        w = tw.widget(index)
-        if w and w.closeRepoWidget():
-            tw.removeTab(index)
-            w.deleteLater()
-            self.updateMenu()
+        if 0 <= index < tw.count():
+            w = tw.widget(index)
+            try:
+                reporoot = w.repo.root
+            except:
+                reporoot = ''
+            if w and w.closeRepoWidget():
+                tw.removeTab(index)
+                w.deleteLater()
+                self.updateMenu()
+                self.lastClosedRepoRootList = [reporoot]
+
+    def reopenLastClosedTabs(self):
+        for reporoot in self.lastClosedRepoRootList:
+            if os.path.isdir(reporoot):
+                self.showRepo(reporoot)
+        self.lastClosedRepoRootList = []
 
     def repoTabChanged(self, index=0):
         w = self.repoTabsWidget.currentWidget()
         if w:
             self.updateHistoryActions()
             self.updateMenu()
-        self.log.setRepository(w and w.repo or None)
+            if w.repo:
+                root = w.repo.root
+                self.activeRepoChanged.emit(hglib.tounicode(root))
+        else:
+            self.activeRepoChanged.emit("")
+        repo = w and w.repo or None
+        self.log.setRepository(repo)
+        self.mqpatches.setrepo(repo)
 
     def addRepoTab(self, repo):
         '''opens the given repo in a new tab'''
@@ -485,16 +645,21 @@ class Workbench(QMainWindow):
         rw.output.connect(self.log.output)
         rw.makeLogVisible.connect(self.log.setShown)
         rw.revisionSelected.connect(self.updateHistoryActions)
-        rw.repoLinkClicked.connect(self.showRepo)
+        rw.repoLinkClicked.connect(self.openLinkedRepo)
         rw.taskTabsWidget.currentChanged.connect(self.updateTaskViewMenu)
         rw.toolbarVisibilityChanged.connect(self.updateToolBarActions)
+        rw.shortNameChanged.connect(self.reporegistry.shortNameChanged)
+        rw.baseNodeChanged.connect(self.reporegistry.baseNodeChanged)
+        rw.repoChanged.connect(self.reporegistry.repoChanged)
 
         tw = self.repoTabsWidget
         index = self.repoTabsWidget.addTab(rw, rw.title())
         tw.setCurrentIndex(index)
         rw.titleChanged.connect(
             lambda title: tw.setTabText(tw.indexOf(rw), title))
-        self.reporegistry.addRepo(repo)
+        rw.showIcon.connect(
+            lambda icon: tw.setTabIcon(tw.indexOf(rw), icon))
+        self.reporegistry.addRepo(repo.root)
 
         self.updateMenu()
 
@@ -505,8 +670,8 @@ class Workbench(QMainWindow):
     def setHistoryColumns(self, *args):
         """Display the column selection dialog"""
         w = self.repoTabsWidget.currentWidget()
-        dlg = ColumnSelectDialog(repomodel.ALLCOLUMNS,
-                                 w and w.repoview.model()._columns)
+        dlg = ColumnSelectDialog('workbench', _('Workbench'),
+                                 w and w.repoview.model() or None)
         if dlg.exec_() == QDialog.Accepted:
             if w:
                 w.repoview.model().updateColumns()
@@ -551,7 +716,7 @@ class Workbench(QMainWindow):
         dlg.finished.connect(dlg.deleteLater)
         if dlg.exec_():
             path = dlg.getPath()
-            self.openRepo(path)
+            self._openRepo(path, False)
 
     def cloneRepository(self):
         """ Run clone dialog """
@@ -559,14 +724,13 @@ class Workbench(QMainWindow):
         repoWidget = self.repoTabsWidget.currentWidget()
         if repoWidget:
             root = repoWidget.repo.root
-            args = [root, os.path.dirname(root)]
+            args = [root, root + '-clone']
         else:
             args = []
         dlg = CloneDialog(args, parent=self)
         dlg.finished.connect(dlg.deleteLater)
-        if dlg.exec_():
-            path = dlg.getDest()
-            self.openRepo(path)
+        dlg.clonedRepository.connect(self.showRepo)
+        dlg.exec_()
 
     def openRepository(self):
         """ Open repo from File menu """
@@ -580,25 +744,22 @@ class Workbench(QMainWindow):
         FD = QFileDialog
         path = FD.getExistingDirectory(self, caption, cwd,
                                        FD.ShowDirsOnly | FD.ReadOnly)
-        self._openRepo(path=hglib.fromunicode(path))
+        self._openRepo(hglib.fromunicode(path), False)
 
-    def _openRepo(self, path, reuse=False):
-        if path and not path.startswith('ssh://'):
+    def _openRepo(self, root, reuse):
+        if root and not root.startswith('ssh://'):
             if reuse:
-                for rw in self._findrepowidget(path):
+                for rw in self._findrepowidget(root):
+                    self.repoTabsWidget.setCurrentWidget(rw)
                     return
             try:
-                repo = thgrepo.repository(path=path)
+                repo = thgrepo.repository(path=root)
                 self.addRepoTab(repo)
             except RepoError:
-                upath = hglib.tounicode(path)
+                upath = hglib.tounicode(root)
                 qtlib.WarningMsgBox(_('Failed to open repository'),
                         _('%s is not a valid repository') % upath)
 
-    def goto(self, root, rev):
-        for rw in self._findrepowidget(root):
-            rw.goto(rev)
-
     def _findrepowidget(self, root):
         """Iterates RepoWidget for the specified root"""
         tw = self.repoTabsWidget
@@ -624,6 +785,10 @@ class Workbench(QMainWindow):
         s.setValue(wb + 'geometry', self.saveGeometry())
         s.setValue(wb + 'windowState', self.saveState())
         s.setValue(wb + 'showPaths', self.actionShowPaths.isChecked())
+        s.setValue(wb + 'showSubrepos', self.actionShowSubrepos.isChecked())
+        s.setValue(wb + 'showNetworkSubrepos',
+            self.actionShowNetworkSubrepos.isChecked())
+        s.setValue(wb + 'showShortPaths', self.actionShowShortPaths.isChecked())
         s.setValue(wb + 'saveRepos', self.actionSaveRepos.isChecked())
         repostosave = []
         if self.actionSaveRepos.isChecked():
@@ -641,11 +806,45 @@ class Workbench(QMainWindow):
         save = s.value(wb + 'saveRepos').toBool()
         self.actionSaveRepos.setChecked(save)
         for path in hglib.fromunicode(s.value(wb + 'openrepos').toString()).split(','):
-            self._openRepo(path)
+            self._openRepo(path, False)
+
+        # Load the repo registry settings. Note that we must allow the
+        # repo registry to assemble itself before toggling its settings
+        # Also the view path setttings should be enabled last, once we have
+        # loaded the repo subrepositories (if needed)
+
+        # Normally, checking the "show subrepos" and the "show network subrepos"
+        # settings will trigger a reload of the repo registry.
+        # To avoid reloading it twice (every time we set one of its view
+        # settings), we tell the setters to avoid reloading the repo tree
+        # model, and then we  manually reload the model
+        ssr = s.value(wb + 'showSubrepos',
+            defaultValue=QVariant(True)).toBool()
+        snsr = s.value(wb + 'showNetworkSubrepos',
+            defaultValue=QVariant(True)).toBool()
+        ssp = s.value(wb + 'showShortPaths',
+            defaultValue=QVariant(True)).toBool()
+        self.reporegistry.setShowSubrepos(ssr, False)
+        self.reporegistry.setShowNetworkSubrepos(snsr, False)
+        self.reporegistry.setShowShortPaths(ssp)
+
+        # Note that calling setChecked will NOT reload the model if the new
+        # setting is the same as the one in the repo registry
+        QTimer.singleShot(0, lambda: self.actionShowSubrepos.setChecked(ssr))
+        QTimer.singleShot(0, lambda: self.actionShowNetworkSubrepos.setChecked(ssr))
+        QTimer.singleShot(0, lambda: self.actionShowShortPaths.setChecked(ssp))
+
+        # Manually reload the model now, to apply the settings
+        self.reporegistry.reloadModel()
+
         # Allow repo registry to assemble itself before toggling path state
         sp = s.value(wb + 'showPaths').toBool()
         QTimer.singleShot(0, lambda: self.actionShowPaths.setChecked(sp))
 
+    def goto(self, root, rev):
+        for rw in self._findrepowidget(root):
+            rw.goto(rev)
+
     def closeEvent(self, event):
         if not self.closeRepoTabs():
             event.ignore()
@@ -672,29 +871,12 @@ class Workbench(QMainWindow):
     def explore(self):
         w = self.repoTabsWidget.currentWidget()
         if w:
-            self.launchExplorer(w.repo.root)
+            QDesktopServices.openUrl(QUrl.fromLocalFile(w.repo.root))
 
     def terminal(self):
         w = self.repoTabsWidget.currentWidget()
         if w:
-            self.launchTerminal(w.repo)
-
-    def launchExplorer(self, root):
-        """open Windows Explorer at the repo root"""
-        QDesktopServices.openUrl(QUrl.fromLocalFile(root))
-
-    def launchTerminal(self, repo):
-        shell = repo.shell()
-        if shell:
-            cwd = os.getcwd()
-            try:
-                os.chdir(repo.root)
-                QProcess.startDetached(shell)
-            finally:
-                os.chdir(cwd)
-        else:
-            qtlib.InfoMsgBox(_('No shell configured'),
-                       _('A terminal shell must be configured'))
+            qtlib.openshell(w.repo.root, w.repo.displayname)
 
     def editSettings(self):
         tw = self.repoTabsWidget
